(* $Id: separate.ml,v 1.4 2014/02/27 16:36:36 fclement Exp $ *)

open Ppm;;
open Pixmap;;

let usage () =
  prerr_endline "\
     Usage: \"separate <file>.ppm\" separates the Red, Green and Blue channels\
    \nof the PPM image <file>.ppm into files <file>-red.ppm, <file>-green.ppm\
    \nand <file>-blue.ppm."
;;

let separate_error s =
  prerr_endline (Printf.sprintf "separate: %s." s);
  usage ();
  exit 2
;;

let ( ^^^ ) = Filename.concat;;

let insert_to_basename fname suf ext =
  let dname, bname = Filename.dirname fname, Filename.basename fname in
  if not (Filename.check_suffix fname ext) then
    separate_error
      (Printf.sprintf "wrong file type, should be a %s image" ext) else
  dname ^^^ Filename.chop_suffix bname ext ^ suf ^ ext
;;

let save_channels fname pxr pxg pxb =
  let rfname = insert_to_basename fname "-red" ".ppm" in
  let gfname = insert_to_basename fname "-green" ".ppm" in
  let bfname = insert_to_basename fname "-blue" ".ppm" in
  save_ppm rfname pxr;
  save_ppm gfname pxg;
  save_ppm bfname pxb
;;

let separate_rgb_channels fname =
  let px = load_ppm fname in
  let pxr = red_pixmap_of_red_channel px in
  let pxg = green_pixmap_of_green_channel px in
  let pxb = blue_pixmap_of_blue_channel px in
  save_channels fname pxr pxg pxb
;;

let main () =
  match Sys.argv with
  | [| _command_name; fname |] ->
    separate_rgb_channels fname
  | _ -> separate_error "wrong number of arguments"
;;

main ();;
